module CL_1ES_1927p654

using QSFit
import QSFit: AbstractSpectralLine, CombinedLine, NarrowLine, BroadLine, BroadBaseLine,
    default_options, qso_cont_component, line_components, known_spectral_lines

abstract type q1927p654 <: DefaultRecipe end

export q1927p654


function default_options(::Type{T}) where T <: q1927p654
    out = default_options(supertype(T))
    out[:use_OIII_5007_bw] = true
    out[:use_broad_Ha_base] = true
    out[:use_host_template] = true
    #out[:host_template] = "S0"
    #out[:wavelength_range][1] = 4000
    out[:n_unk] = 0
    return out
end


function qso_cont_component(::Type{T}) where T <: q1927p654
    comp = qso_cont_component(supertype(T))
    comp.alpha.low  = -5
    comp.alpha.high =  5
    return comp
end


function known_spectral_lines(::Type{T}) where T <: q1927p654
    list = Vector{AbstractSpectralLine}()
    #push!(list,CombinedLine(  :Lyb          , 1026.0  ))
    push!(list, CombinedLine(  :Lya          , 1215.24 ))
    push!(list, NarrowLine(    :NV_1241      , 1240.81 ))
    push!(list, BroadLine(     :OI_1306      , 1305.53 ))
    push!(list, BroadLine(     :CII_1335     , 1335.31 ))
    push!(list, BroadLine(     :SiIV_1400    , 1399.8  ))
    push!(list, CombinedLine(  :CIV_1549     , 1549.48 ))
    #push!(list,BroadLine(     :HeII         , 1640.4  ))
    #push!(list,BroadLine(     :OIII         , 1665.85 ))
    #push!(list,BroadLine(     :AlIII        , 1857.4  ))
    push!(list, BroadLine(     :CIII_1909    , 1908.734))
    push!(list, BroadLine(     :CII          , 2326.0  ))
    push!(list, BroadLine(     :F2420        , 2420.0  ))
    push!(list, CombinedLine(  :MgII_2798    , 2799.117))
    #push!(list,NarrowLine(    :NeVN         , 3346.79 ))
    push!(list, NarrowLine(    :NeVI_3426    , 3426.85 ))
    push!(list, NarrowLine(    :OII_3727     , 3729.875))
    push!(list, NarrowLine(    :NeIII_3869   , 3869.81 ))
    #push!(list,BroadLine(     :Hd           , 4102.89 ))
    push!(list, BroadLine(     :Hg           , 4341.68 ))
    push!(list, NarrowLine(    :OIII_4363    , 4363.00 ))  # TODO: Check wavelength is correct
    push!(list, BroadLine(     :BOH          , 4490.   ))
    push!(list, BroadLine(     :HeII         , 4686.   ))
    push!(list, CombinedLine(  :Hb           , 4862.68 ))
    push!(list, BroadBaseLine( :Hb_base      , 4862.68 ))
    push!(list, NarrowLine(    :OIII_4959    , 4960.295))
    push!(list, NarrowLine(    :OIII_5007    , 5008.240))
    push!(list, NarrowLine(    :OIII_5007_bw , 5008.240))
    push!(list, BroadLine(     :HeI_5876     , 5877.30 ))
    push!(list, NarrowLine(    :OI_6300      , 6300.00 ))  # TODO: Check wavelength is correct
    push!(list, NarrowLine(    :OI_6364      , 6364.00 ))  # TODO: Check wavelength is correct
    push!(list, NarrowLine(    :NII_6549     , 6549.86 ))
    push!(list, CombinedLine(  :Ha           , 6564.61 ))
    push!(list, BroadBaseLine( :Ha_base      , 6564.61 ))
    push!(list, NarrowLine(    :NII_6583     , 6585.27 ))
    push!(list, NarrowLine(    :SII_6716     , 6718.29 ))
    push!(list, NarrowLine(    :SII_6731     , 6732.67 ))
    return list
end


function line_components(::Type{T}, line::QSFit.CombinedLine) where T <: q1927p654
    br = QSFit.SpecLineAsymmGauss(line.λ)
    br.fwhm.val  = 5e3
    br.fwhm.low  = 900
    br.fwhm.high = 1.5e4
    br.voff.low  = -3e3
    br.voff.high =  3e3

    na = QSFit.SpecLineGauss(line.λ)
    na.fwhm.val  = 5e2
    na.fwhm.low  = 100
    na.fwhm.high = 1e3
    na.voff.low  = -1e3
    na.voff.high =  1e3

    return [Symbol(:br_, line.name) => br
            Symbol(:na_, line.name) => na]
end

end
